/*
 *  Vazaar
 *  file: init.c
 *
 *
 *  Copyright (C) 2005 Tomás Vírseda <kaskaras@yahoo.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <gconf/gconf-client.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libxml/xmlreader.h>
#include <glade/glade.h>
#include <libgnome/libgnome.h>


#include "init.h"
//#include "e-splash.h"
#include "support.h"
#include "vazaar.h"
#include "categories.h"
#include "items.h"
#include "callbacks.h"
#include "debug.h"


extern xmlDocPtr vazaar_doc;
extern xmlDocPtr cats_doc;
extern xmlDocPtr items_doc;
extern xmlDocPtr types_doc;
xmlNodePtr  node_found;


Vazaar *
init_vazaar 						(	gint, 
										gchar *[]);
void
init_gui							(	Vazaar *app);

void
load_config							();

void
load_data							(	Vazaar *app);

void
fix_bugs							(	Vazaar *app);

void
check_and_fix_system_categories_bug (	Vazaar *app);

GtkWidget*
create_splash						(void);

void
init_libs (int argc, char *argv[])
{
	g_print ("Initializing libs... \n");
	
    /*
     * Initialize GTK 
     */
    gtk_init(&argc, &argv);
	
	/*
	 * Setting environment
	 */
	#ifdef ENABLE_NLS
		bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
		bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
		textdomain(GETTEXT_PACKAGE);
	#endif

    /*
     * Initialize GNOME Core Libs 
     */
    gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
		       argc, argv,
		       GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
	
    /*
     * Initialize GConf libs
     */	
	
	
    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION if (DEBUG)
	
    /*
     * The GNOME virtual file-system library 
     */
    gnome_vfs_init();
}

void
load_config ()
{
	guint			perm;
    gchar          *vazaar_path,
                   *items_path,
                   *cats_path;
    GnomeVFSResult  res;
    xmlNodePtr      vazaar_root,
                    items_root,
                    cats_root;
	gboolean		success;

	g_print ("Loading config...\n");
	success = FALSE;
	perm = 	GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_USER_EXEC;
	
    vazaar_path = vazaar_xml_vazaardoc_get();
    items_path = vazaar_xml_itemdoc_get();
    cats_path = vazaar_xml_catdoc_get();

    /*
     * ¿Existe el fichero de datos? 
     */
    if (!g_file_test(vazaar_path, G_FILE_TEST_IS_REGULAR)) {
		/*
		 * Creamos el directorio de la aplicacion 
		 */
		if (!g_file_test(vazaar_cmd_appdir_get(), G_FILE_TEST_IS_DIR)) {
			res = gnome_vfs_make_directory(vazaar_cmd_appdir_get(), perm);
			/*
			 * Y tambien el directorio para almacenar los datos 
			 */
			res = gnome_vfs_make_directory(vazaar_cmd_storedir_get(), perm);
		}
		// vazaar.xml
		// vazaar_xml_vazaar_create ();
		vazaar_doc = xmlNewDoc("1.0");
		vazaar_root = xmlNewDocNode(vazaar_doc, NULL, "vazaar", NULL);
		xmlDocSetRootElement(vazaar_doc, vazaar_root);
		xmlSaveFile(vazaar_path, vazaar_doc);
	
		
		// items.xml
		items_doc = xmlNewDoc("1.0");
		items_root = xmlNewDocNode(items_doc, NULL, "items", NULL);
		xmlDocSetRootElement(items_doc, items_root);
		xmlSaveFile(items_path, items_doc);
	
		// categories.xml
		cats_doc = xmlNewDoc("1.0");
		cats_root = xmlNewDocNode(cats_doc, NULL, "categories", NULL);
		xmlDocSetRootElement(cats_doc, cats_root);
//		vazaar_xml_cat_add_system (cats_root);
		xmlSaveFile(cats_path, cats_doc);
    } else {
		vazaar_doc = xmlParseFile(vazaar_path);
		items_doc = xmlParseFile(items_path);
		cats_doc = xmlParseFile(cats_path);
    }

    if ((vazaar_doc) && (items_doc) && (cats_doc)) {
		success = TRUE;
    }

    g_free(vazaar_path);
    g_free(cats_path);
    g_free(items_path);
}

Vazaar *
init_vazaar (int argc, char *argv[]) {
	GtkWidget	*splash = NULL;
	Vazaar *app = NULL;
	init_libs (argc, argv);	
	splash = create_splash ();
	gtk_widget_show (splash);
	while (gtk_events_pending ())
	  gtk_main_iteration ();
	g_usleep (G_USEC_PER_SEC);
	load_config ();
	fix_bugs (app);
	app = vazaar_gui_vazaar_new ();
	/* check app */
	if (app) {
//		convert_from_old_versions ();
		load_data (app);
		init_gui (app);	
	}
	g_print ("Starting GNOME Vazaar...\n\n");
	//g_usleep (G_USEC_PER_SEC);
	while (gtk_events_pending ())
	  gtk_main_iteration ();
	gtk_widget_destroy (splash);
}

void
fix_bugs (Vazaar *app) {
	/* There is a bug until 0.2.2 version with unstralated system categories 
		This problem fix the probem */
	g_print ("Fixing bugs... if these exist...\n");
	check_and_fix_system_categories_bug (app);
}

void
load_data (Vazaar *app) {
	GtkMozEmbed		*gecko;
	
	g_print ("Loading data...\n");
	
	/* Add mime icons to cache */
	vazaar_cmd_mimeiconscache_fill (app);
	
	/* Load a HTML document to initialize gecko */
    gecko = (GtkMozEmbed *) gtk_moz_embed_new();
    gtk_moz_embed_load_url(gecko, "about:mozilla");
	g_free (gecko);
}

void
init_gui(Vazaar *app)
{
    GtkTreeModel   *model;
	gint			width, height;
	
	g_print ("Loading GUI...\n");
    
    vazaar_gui_cat_load(app);
    model = vazaar_gui_itemmodel_create();
    vazaar_gui_itemmodel_createview(model, GTK_TREE_VIEW (app->treeitem));
	vazaar_gui_vazaar_show(app);
}


void
vazaar_xml_cat_add_system (xmlNodePtr root)
{
	xmlNodePtr child;
	
    child = xmlNewChild(root, NULL, "category", NULL);
    xmlSetProp(child, "icon", "generic.png");
    xmlSetProp(child, "name", _("Generic"));
    xmlSetProp(child, "desc", _(""));
	xmlSetProp(child, "syst", "false");

    child = xmlNewChild(root, NULL, "category", NULL);
    xmlSetProp(child, "icon", "important.png");
    xmlSetProp(child, "name", _("Important"));
    xmlSetProp(child, "desc", _(""));
	xmlSetProp(child, "syst", "false");

	child = xmlNewChild(root, NULL, "category", NULL);
	xmlSetProp(child, "icon", "favorite.png");
	xmlSetProp(child, "name", _("Favorites"));
	xmlSetProp(child, "desc", _(""));
	xmlSetProp(child, "syst", "false");
}

void
check_and_fix_system_categories_bug (Vazaar *app) {
    xmlNodePtr       root,
                     child;
	gint			 cmp = -1;
	VazaarCat		*cat;
	
    node_found = NULL;

    root = xmlDocGetRootElement(cats_doc);
    vazaar_xml_cat_find(root->xmlChildrenNode, "Wastebasket");
    child = node_found;
	if (child) {
		cat = vazaar_xml_cat_get (child);
		if (g_utf8_collate (cat->syst, "true") == 0) {
				g_print ("\n\tFound bug. Fixing...");
				vazaar_xml_cat_del ("Wastebasket", TRUE);
				vazaar_xml_cat_del ("Generic", TRUE);
				vazaar_xml_cat_del ("Favorites", TRUE);
				vazaar_xml_cat_del ("Important", TRUE);
				vazaar_xml_cat_add_system (root);
				vazaar_xml_cats_savefile ();
				g_print (" OK\n");
		}
	}
}

GtkWidget*
create_splash (void)
{
	GtkWidget	*window1;
	GtkWidget	*image1;
	gchar		*splashpath;
	
	splashpath = g_strdup_printf ("%s/%s", PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/system", "vazaar_splash.png");
	window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window1, 300, 200);
	gtk_window_set_title (GTK_WINDOW (window1), _("splash"));
	gtk_window_set_position (GTK_WINDOW (window1), GTK_WIN_POS_CENTER_ON_PARENT);
	gtk_window_set_default_size (GTK_WINDOW (window1), 300, 200);
	gtk_window_set_resizable (GTK_WINDOW (window1), FALSE);
	gtk_window_set_decorated (GTK_WINDOW (window1), FALSE);
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window1), TRUE);
	gtk_window_set_type_hint (GTK_WINDOW (window1), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
	
	image1 = create_pixmap (window1, splashpath);
	gtk_widget_show (image1);
	gtk_container_add (GTK_CONTAINER (window1), image1);
	
	return window1;
}
